@@ -6,7 +6,7 @@ Arbor class and member functions
 """
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2016-2017, Britton Smith <brittonsmith@gmail.com>
+# Copyright (c) ytree development team. All rights reserved.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -26,8 +26,8 @@ from yt.extern.six import \
 from yt.frontends.ytdata.utilities import \
     save_as_dataset
 from yt.funcs import \
-    ensure_dir, \
-    get_pbar
+    get_pbar, \
+    TqdmProgressBar
 from yt.units.dimensions import \
     length
 from yt.units.unit_registry import \
@@ -42,8 +42,10 @@ from ytree.arbor.fields import \
     FakeFieldContainer, \
     FieldContainer, \
     FieldInfoContainer
+from ytree.arbor.misc import \
+    _determine_output_filename
 from ytree.arbor.io import \
-    FallbackRootFieldIO, \
+    DefaultRootFieldIO, \
     TreeFieldIO
 from ytree.arbor.tree_node import \
     TreeNode
@@ -84,7 +86,7 @@ class Arbor(object):
     """
 
     _field_info_class = FieldInfoContainer
-    _root_field_io_class = FallbackRootFieldIO
+    _root_field_io_class = DefaultRootFieldIO
     _tree_field_io_class = TreeFieldIO
 
     def __init__(self, filename):
@@ -94,14 +96,21 @@ class Arbor(object):
 
         self.filename = filename
         self.basename = os.path.basename(filename)
-        self.unit_registry = UnitRegistry()
         self._parse_parameter_file()
         self._set_units()
-        self._root_field_data = FieldContainer(self)
-        self._setup_fields()
-        self._set_default_selector()
+        self._field_data = FieldContainer(self)
         self._node_io = self._tree_field_io_class(self)
         self._root_io = self._root_field_io_class(self)
+        self._get_data_files()
+        self._setup_fields()
+        self._set_default_selector()
+
+    def _get_data_files(self):
+        """
+        Get all files that hold field data and make them known
+        to the i/o system.
+        """
+        pass
 
     def _parse_parameter_file(self):
         """
@@ -119,14 +128,15 @@ class Arbor(object):
     def is_setup(self, tree_node):
         """
         Return True if arrays of uids and descendent uids have
-        been read in.
+        been read in. Setup has also completed if tree is already
+        grown.
         """
-        return tree_node.root != -1 or \
+        return self.is_grown(tree_node) or \
           tree_node._uids is not None
 
     def _setup_tree(self, tree_node, **kwargs):
         """
-        Create arrays of uids and descids and attach them to the
+        Create arrays of uids and desc_uids and attach them to the
         root node.
         """
         # skip if this is not a root or if already setup
@@ -141,7 +151,7 @@ class Arbor(object):
         field_data  = self._node_io._read_fields(tree_node, fields,
                                                  dtypes=dtypes, **kwargs)
         tree_node._uids      = field_data[halo_id_f]
-        tree_node._descids   = field_data[desc_id_f]
+        tree_node._desc_uids = field_data[desc_id_f]
         tree_node._tree_size = tree_node._uids.size
 
     def is_grown(self, tree_node):
@@ -149,9 +159,9 @@ class Arbor(object):
         Return True if a tree has been fully assembled, i.e.,
         the hierarchy of ancestor tree nodes has been built.
         """
-        return hasattr(tree_node, "treeid")
+        return tree_node.root != -1
 
-    def _grow_tree(self, tree_node):
+    def _grow_tree(self, tree_node, **kwargs):
         """
         Create an array of TreeNodes hanging off the root node
         and assemble the tree structure.
@@ -160,7 +170,7 @@ class Arbor(object):
         if self.is_grown(tree_node):
             return
 
-        self._setup_tree(tree_node)
+        self._setup_tree(tree_node, **kwargs)
         nhalos   = tree_node.uids.size
         nodes    = np.empty(nhalos, dtype=np.object)
         nodes[0] = tree_node
@@ -168,32 +178,118 @@ class Arbor(object):
             nodes[i] = TreeNode(tree_node.uids[i], arbor=self)
         tree_node._nodes = nodes
 
-        uidmap   = {}
+        # Add tree information to nodes
+        uidmap = {}
         for i, node in enumerate(nodes):
             node.treeid = i
             node.root   = tree_node
-            descid      = tree_node.descids[i]
             uidmap[tree_node.uids[i]] = i
+
+        # Link ancestor/descendents
+        # Separate loop for trees like lhalotree where descendent
+        # can follow in order
+        for i, node in enumerate(nodes):
+            descid      = tree_node.desc_uids[i]
             if descid != -1:
-                desc = nodes[uidmap[tree_node.descids[i]]]
+                desc = nodes[uidmap[descid]]
                 desc.add_ancestor(node)
                 node.descendent = desc
 
     def _node_io_loop(self, func, *args, **kwargs):
+        """
+        Call the provided function over a list of nodes.
+
+        If possible, group nodes by common data files to speed
+        things up.  This should work like __iter__, except we call
+        a function instead of yielding.
+
+        Parameters
+        ----------
+        func : function
+            Function to be called on an array of nodes.
+        pbar : optional, string or yt.funcs.TqdmProgressBar
+            A progress bar to be updated with each iteration.
+            If a string, a progress bar will be created and the
+            finish function will be called. If a progress bar is
+            provided, the finish function will not be called.
+            Default: None (no progress bar).
+        root_nodes : optional, array of root TreeNodes
+            Array of nodes over which the function will be called.
+            If None, the list will be self.trees (i.e., all
+            root_nodes).
+            Default: None.
+        store : optional, string
+            If not None, any return value captured from the function
+            will be stored in an attribute with this name associated
+            with the TreeNode.
+            Default: None.
+        """
+
+        pbar = kwargs.pop("pbar", None)
         root_nodes = kwargs.pop("root_nodes", None)
         if root_nodes is None:
             root_nodes = self.trees
-        pbar = kwargs.pop("pbar", None)
-        if pbar is None:
-            mypbar = fake_pbar
-        else:
-            mypbar = get_pbar
+        store = kwargs.pop("store", None)
+        data_files, node_list = self._node_io_loop_prepare(root_nodes)
+        nnodes = sum([nodes.size for nodes in node_list])
 
-        pbar = mypbar(pbar, len(root_nodes))
-        for node in root_nodes:
-            func(node, *args, **kwargs)
-            pbar.update(1)
-        pbar.finish()
+        finish = True
+        if pbar is None:
+            pbar = fake_pbar("", nnodes)
+        elif not isinstance(pbar, TqdmProgressBar):
+            pbar = get_pbar(pbar, nnodes)
+        else:
+            finish = False
+
+        for data_file, nodes in zip(data_files, node_list):
+            self._node_io_loop_start(data_file)
+            for node in nodes:
+                rval = func(node, *args, **kwargs)
+                if store is not None:
+                    setattr(node, store, rval)
+                pbar.update(1)
+            self._node_io_loop_finish(data_file)
+
+        if finish:
+            pbar.finish()
+
+    def _node_io_loop_start(self, data_file):
+        pass
+
+    def _node_io_loop_finish(self, data_file):
+        pass
+
+    def _node_io_loop_prepare(self, root_nodes):
+        """
+        This is called at the beginning of _node_io_loop.
+
+        In different frontends, this can be used to group nodes by
+        common data files.
+
+        Return
+        ------
+        list of data files and a list of node arrays
+
+        Each data file corresponds to an array of nodes.
+        """
+
+        return [None], [root_nodes]
+
+    def __iter__(self):
+        """
+        Iterate over all items in the tree list.
+
+        If possible, group nodes by common data files to speed
+        things up.
+        """
+
+        data_files, node_list = self._node_io_loop_prepare(self.trees)
+
+        for data_file, nodes in zip(data_files, node_list):
+            self._node_io_loop_start(data_file)
+            for node in nodes:
+                yield node
+            self._node_io_loop_finish(data_file)
 
     _trees = None
     @property
@@ -209,26 +305,23 @@ class Arbor(object):
         return self.basename
 
     def __getitem__(self, key):
+        return self.query(key)
+
+    def query(self, key):
         """
         If given a string, return an array of field values for the
         roots of all trees.
         If given an integer, return a tree from the list of trees.
-
         """
         if isinstance(key, string_types):
             if key in ("tree", "prog"):
                 raise SyntaxError("Argument must be a field or integer.")
             self._root_io.get_fields(self, fields=[key])
-            return self._root_field_data[key]
+            if self.field_info[key].get("type") == "analysis":
+                return self._field_data.pop(key)
+            return self._field_data[key]
         return self.trees[key]
 
-    def __iter__(self):
-        """
-        Iterate over all items in the tree list.
-        """
-        for t in self.trees:
-            yield t
-
     def __len__(self):
         """
         Return length of tree list.
@@ -259,6 +352,8 @@ class Arbor(object):
         """
         Unit system registry.
         """
+        if self._unit_registry is None:
+            self._unit_registry = UnitRegistry()
         return self._unit_registry
 
     @unit_registry.setter
@@ -301,8 +396,10 @@ class Arbor(object):
     def _setup_fields(self):
         self.derived_field_list = []
         self.analysis_field_list = []
+        self.field_info.setup_known_fields()
         self.field_info.setup_aliases()
         self.field_info.setup_derived_fields()
+        self.field_info.setup_vector_fields()
 
     def _set_units(self):
         """
@@ -383,6 +480,48 @@ class Arbor(object):
                      fields=None):
         """
         Select halos from the arbor based on a set of criteria given as a string.
+
+
+        Parameters
+        ----------
+
+        criteria: string
+            A string that will eval to a Numpy-like selection operation
+            performed on a TreeNode object called "tree".
+            Example: 'tree["tree", "redshift"] > 1'
+        trees : optional, list or array of TreeNodes
+            A list or array of TreeNode objects in which to search. If none given,
+            the search is performed over the full arbor.
+        select_from : optional, "tree" or "prog"
+            Determines whether to perform the search over the full tree or just
+            the main progenitors. Note, the value given must be consistent with
+            what appears in the criteria string. For example, a criteria
+            string of 'tree["tree", "redshift"] > 1' cannot be used when setting
+            select_from to "prog".
+            Default: "tree".
+        fields : optional, list of strings
+            Use to provide a list of fields required by the criteria evaluation.
+            If given, fields will be preloaded in an optimized way and the search
+            will go faster.
+            Default: None.
+
+        Returns
+        -------
+
+        halos : array of TreeNodes
+            A flat array of all TreeNodes meeting the criteria.
+
+        Examples
+        --------
+
+        >>> import ytree
+        >>> a = ytree.load("tree_0_0_0.dat")
+        >>> halos = a.select_halos('tree["tree", "redshift"] > 1',
+        ...                        fields=["redshift"])
+        >>>
+        >>> halos = a.select_halos('tree["prog", "mass"].to("Msun") >= 1e10',
+        ...                        select_from="prog", fields=["mass"])
+
         """
 
         if select_from not in ["tree", "prog"]:
@@ -407,7 +546,12 @@ class Arbor(object):
         halos = []
         pbar = get_pbar("Selecting halos", self.trees.size)
         for tree in trees:
-            my_filter = eval(criteria)
+            my_filter = np.asarray(eval(criteria))
+            if my_filter.size != tree[select_from].size:
+                raise RuntimeError(
+                    ("Filter array and tree array sizes do not match. " +
+                     "Make sure select_from (\"%s\") matches criteria (\"%s\").") %
+                    (select_from, criteria))
             halos.extend(tree[select_from][my_filter])
             pbar.update(1)
         pbar.finish()
@@ -504,7 +648,7 @@ class Arbor(object):
 
     def add_derived_field(self, name, function,
                           units=None, description=None,
-                          force_add=True):
+                          vector_field=False, force_add=True):
         r"""
         Add a field that is a function of other fields.
 
@@ -521,6 +665,9 @@ class Arbor(object):
             The units in which the field will be returned.
         description : optional, string
             A short description of the field.
+        vector_field: optional, bool
+            If True, field is an xyz vector.
+            Default: False.
         force_add : optional, bool
             If True, add field even if it already exists and warn the
             user and raise an exception if dependencies do not exist.
@@ -532,7 +679,7 @@ class Arbor(object):
 
         >>> import ytree
         >>> a = ytree.load("tree_0_0_0.dat")
-        >>> def _redshift(arbor, data):
+        >>> def _redshift(field, data):
         ...     return 1. / data["scale"] - 1
         ...
         >>> a.add_derived_field("redshift", _redshift)
@@ -556,20 +703,37 @@ class Arbor(object):
 
         if units is None:
             units = ""
+        info = {"name": name, "type": "derived", "function": function,
+                "units": units, "vector_field": vector_field,
+                "description": description}
+
         fc = FakeFieldContainer(self, name=name)
         try:
-            rv = function(fc)
+            rv = function(info, fc)
+        except TypeError as e:
+            raise RuntimeError(
+"""
+
+Field function syntax in ytree has changed. Field functions must
+now take two arguments, as in the following:
+def my_field(field, data):
+    return data['mass']
+
+Check the TypeError exception above for more details.
+""")
+            raise e
+
         except ArborFieldDependencyNotFound as e:
             if force_add:
                 raise e
             else:
                 return
+
         rv.convert_to_units(units)
+        info["dependencies"] = list(fc.keys())
+
         self.derived_field_list.append(name)
-        self.field_info[name] = \
-          {"type": "derived", "function": function,
-           "units": units, "description": description,
-           "dependencies": list(fc.keys())}
+        self.field_info[name] = info
 
     @classmethod
     def _is_valid(cls, *args, **kwargs):
@@ -589,8 +753,9 @@ class Arbor(object):
         Parameters
         ----------
         filename : optional, string
-            Output file keyword.  Main header file will be named
-            <filename>/<filename>.h5.
+            Output file keyword.  If filename ends in ".h5",
+            the main header file will be just that.  If not,
+            filename will be <filename>/<basename>.h5.
             Default: "arbor".
         fields : optional, list of strings
             The fields to be saved.  If not given, all
@@ -598,7 +763,7 @@ class Arbor(object):
 
         Returns
         -------
-        filename : string
+        header_filename : string
             The filename of the saved arbor.
 
         Examples
@@ -655,7 +820,8 @@ class Arbor(object):
 
         self._node_io_loop(self._setup_tree, root_nodes=roots,
                            pbar="Setting up trees")
-        self._root_io.get_fields(self, fields=fields)
+        if all_trees:
+            self._root_io.get_fields(self, fields=fields)
 
         # determine file layout
         nn = 0 # node count
@@ -692,7 +858,7 @@ class Arbor(object):
                    for key in ["units", "description"]
                    if key in fi)
             if all_trees:
-                rdata[fieldname] = self._root_field_data[field]
+                rdata[fieldname] = self._field_data[field]
             else:
                 rdata[fieldname] = self.arr([t[field] for t in trees])
             rtypes[fieldname] = "data"
@@ -710,8 +876,8 @@ class Arbor(object):
         htypes = dict((f, "index") for f in hdata)
         htypes.update(rtypes)
 
-        ensure_dir(filename)
-        header_filename = os.path.join(filename, "%s.h5" % filename)
+        filename = _determine_output_filename(filename, ".h5")
+        header_filename = "%s.h5" % filename
         save_as_dataset(ds, header_filename, hdata,
                         field_types=htypes,
                         extra_attrs=extra_attrs)
@@ -734,7 +900,7 @@ class Arbor(object):
             for field, fieldname in zip(fields, fieldnames):
                 for di, node in enumerate(my_nodes):
                     if node.is_root:
-                        ndata = node._tree_field_data[field]
+                        ndata = node._field_data[field]
                     else:
                         ndata = node["tree", field]
                         if field == "desc_uid":
@@ -752,8 +918,7 @@ class Arbor(object):
                       "tree_end_index",
                       "tree_size"]:
                 ftypes[ft] = "index"
-            my_filename = os.path.join(
-                filename, "%s_%04d.h5" % (filename, i))
+            my_filename = "%s_%04d.h5" % (filename, i)
             save_as_dataset({}, my_filename, fdata,
                             field_types=ftypes)
 
@@ -770,8 +935,6 @@ class CatalogArbor(Arbor):
     _data_file_class = None
 
     def __init__(self, filename):
-        self.filename = filename
-        self._get_data_files()
         super(CatalogArbor, self).__init__(filename)
         if "uid" not in self.field_list:
             for field in "uid", "desc_uid":
@@ -783,14 +946,18 @@ class CatalogArbor(Arbor):
         raise NotImplementedError
 
     def _plant_trees(self):
-        fields, _ = \
-          self.field_info.resolve_field_dependencies(["halo_id", "desc_id"])
+        # this can be called once with the list, but fields are
+        # not guaranteed to be returned in order.
+        fields = \
+          [self.field_info.resolve_field_dependencies([field])[0][0]
+           for field in ["halo_id", "desc_id"]]
         halo_id_f, desc_id_f = fields
         dtypes = dict((field, np.int64) for field in fields)
         uid = 0
         trees = []
         nfiles = len(self.data_files)
         descs = lastids = None
+        pbar = get_pbar("Planting trees", len(self.data_files))
         for i, dfl in enumerate(self.data_files):
             if not isinstance(dfl, list):
                 dfl = [dfl]
@@ -807,16 +974,17 @@ class CatalogArbor(Arbor):
                 for it in range(nhalos):
                     descid = data[desc_id_f][it]
                     root = i == 0 or descid == -1
+                    # The data says a descendent exists, but it's not there.
+                    # This shouldn't happen, but it does sometimes.
+                    if not root and descid not in lastids:
+                        root = True
+                        descid = data[desc_id_f][it] = -1
                     tree_node = TreeNode(uid, arbor=self, root=root)
                     tree_node._fi = it
                     tree_node.data_file = data_file
                     batch[it] = tree_node
                     if root:
                         trees.append(tree_node)
-                        if self.field_info["uid"]["source"] == "arbor":
-                            tree_node._root_field_data["uid"] = \
-                              tree_node.uid
-                            tree_node._root_field_data["desc_uid"] = -1
                     else:
                         ancs[descid].append(tree_node)
                     uid += 1
@@ -841,6 +1009,8 @@ class CatalogArbor(Arbor):
                     descs[ib:ib+bs] = batch
                     lastids[ib:ib+bs] = hid
                     ib += bs
+            pbar.update(i)
+        pbar.finish()
 
         self._trees = np.array(trees)
 
@@ -848,37 +1018,34 @@ class CatalogArbor(Arbor):
         if self.is_setup(tree_node):
             return
 
-        nodes   = []
-        uids    = []
-        descids = [-1]
+        nodes     = []
+        uids      = []
+        desc_uids = [-1]
         for i, node in enumerate(tree_node.twalk()):
             node.treeid = i
             node.root   = tree_node
             nodes.append(node)
             uids.append(node.uid)
             if i > 0:
-                descids.append(node.descendent.uid)
+                desc_uids.append(node.descendent.uid)
         tree_node._nodes     = np.array(nodes)
         tree_node._uids      = np.array(uids)
-        tree_node._descids   = np.array(descids)
+        tree_node._desc_uids = np.array(desc_uids)
         tree_node._tree_size = tree_node._uids.size
         # This should bypass any attempt to get this field in
         # the conventional way.
         if self.field_info["uid"]["source"] == "arbor":
-            tree_node._tree_field_data["uid"] = tree_node._uids
-            tree_node._tree_field_data["desc_uid"] = tree_node._descids
+            tree_node._field_data["uid"] = tree_node._uids
+            tree_node._field_data["desc_uid"] = tree_node._desc_uids
 
     def _grow_tree(self, tree_node):
         pass
 
-    def is_grown(self, tree_node):
-        return True
-
 
 global load_warn
 load_warn = True
 
-def load(filename, method=None):
+def load(filename, method=None, **kwargs):
     """
     Load an Arbor, determine the type automatically.
 
@@ -891,6 +1058,9 @@ def load(filename, method=None):
         ConsistentTrees, Rockstar, TreeFarm, YTree.  If not
         given, the type will be determined based on characteristics
         of the input file.
+    kwargs : optional, dict
+        Additional keyword arguments are passed to _is_valid and
+        the determined type.
 
     Returns
     -------
@@ -908,6 +1078,11 @@ def load(filename, method=None):
     >>> a = ytree.load("rockstar_halos/out_0.list")
     >>> # TreeFarm catalogs
     >>> a = ytree.load("my_halos/fof_subhalo_tab_025.0.h5")
+    >>> # LHaloTree catalogs
+    >>> a = ytree.load("my_halos/trees_063.0")
+    >>> # Amiga Halo Finder
+    >>> a = ytree.load("ahf_halos/snap_N64L16_000.parameter",
+    ...                hubble_constant=0.7)
 
     """
     if not os.path.exists(filename):
@@ -915,7 +1090,7 @@ def load(filename, method=None):
     if method is None:
         candidates = []
         for candidate, c in arbor_registry.items():
-            if c._is_valid(filename):
+            if c._is_valid(filename, **kwargs):
                 candidates.append(candidate)
         if len(candidates) == 0:
             raise IOError("Could not determine arbor type for %s." % filename)
@@ -933,7 +1108,7 @@ def load(filename, method=None):
                           (method, arbor_registry.keys()))
 
     global load_warn
-    if method != "YTree" and load_warn:
+    if method not in ["YTree", "LHaloTree"] and load_warn:
         print(
             ("Performance will be improved by saving this arbor with " +
              "\"save_arbor\" and reloading:\n" +
@@ -941,4 +1116,4 @@ def load(filename, method=None):
              "\t>>> fn = a.save_arbor()\n" +
              "\t>>> a = ytree.load(fn)") % filename)
         load_warn = False
-    return arbor_registry[method](filename)
+    return arbor_registry[method](filename, **kwargs)
