@@ -1,12 +1,12 @@
 """
-API for yt.frontends
+API for ytree frontends
 
 
 
 """
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013, yt Development Team.
+# Copyright (c) ytree development team. All rights reserved.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -16,8 +16,10 @@ API for yt.frontends
 import importlib
 
 _frontends = [
+    "ahf",
     "arborarbor",
     "consistent_trees",
+    "lhalotree",
     "rockstar",
     "tree_farm",
     "ytree",
