@@ -6,7 +6,7 @@ TreeFarm class and member functions
 """
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2016, Britton Smith <brittonsmith@gmail.com>
+# Copyright (c) ytree development team. All rights reserved.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -16,8 +16,6 @@ TreeFarm class and member functions
 import numpy as np
 import os
 
-from yt.convenience import \
-    load as yt_load
 from yt.frontends.ytdata.utilities import \
     save_as_dataset
 from yt.funcs import \
@@ -40,6 +38,8 @@ from ytree.tree_farm.ancestry_short import \
 from ytree.tree_farm.halo_selector import \
     selector_registry, \
     clear_id_cache
+from ytree.utilities.io import \
+    yt_load
 from ytree.utilities.logger import \
     set_parallel_logger, \
     ytreeLogger as mylog
@@ -102,14 +102,6 @@ class TreeFarm(object):
 
     """
     def __init__(self, time_series, setup_function=None):
-        from distutils.version import LooseVersion
-        import yt
-        if LooseVersion(yt.__version__) < LooseVersion("3.4"):
-            raise RuntimeError(
-                ("TreeFarm currently requires the development version of yt.\n" +
-                 "Visit http://yt-project.org/#getyt for information on " +
-                 "installing yt from source."))
-
         self.ts = time_series
         self.setup_function = setup_function
 
