@@ -6,7 +6,7 @@ testing utilities
 """
 
 #-----------------------------------------------------------------------------
-# Copyright (c) 2016-2017, Britton Smith <brittonsmith@gmail.com>
+# Copyright (c) ytree development team. All rights reserved.
 #
 # Distributed under the terms of the Modified BSD License.
 #
@@ -14,6 +14,7 @@ testing utilities
 #-----------------------------------------------------------------------------
 
 import h5py
+import numpy as np
 from numpy.testing import \
     assert_array_equal
 import os
@@ -23,6 +24,8 @@ from unittest import \
     TestCase
 from yt.testing import \
     assert_rel_equal
+from yt.funcs import \
+    get_pbar
 
 from ytree.arbor.arbor import \
     load
@@ -73,6 +76,7 @@ class ArborTest(object):
 
     arbor_type = None
     test_filename = None
+    tree_skip = 1
 
     _arbor = None
     @property
@@ -92,9 +96,31 @@ class ArborTest(object):
         assert isinstance(self.arbor, self.arbor_type)
 
     def test_save_and_reload(self):
-        save_and_compare(self.arbor)
+        save_and_compare(self.arbor, skip=self.tree_skip)
 
-def save_and_compare(arbor):
+    def test_vector_fields(self):
+        a = self.arbor
+        t = a[0]
+        for field in a.field_info.vector_fields:
+
+            magfield = np.sqrt((a[field]**2).sum(axis=1))
+            assert_array_equal(a["%s_magnitude" % field], magfield)
+
+            for i, ax in enumerate("xyz"):
+                assert_array_equal(
+                    a["%s_%s" % (field, ax)],
+                    a[field][:, i])
+
+                assert_array_equal(
+                    t["%s_%s" % (field, ax)],
+                    t[field][i])
+
+                for group in ["prog", "tree"]:
+                    assert_array_equal(
+                        t[group, "%s_%s" % (field, ax)],
+                        t[group, field][:, i])
+
+def save_and_compare(arbor, skip=1):
     """
     Check that arbor saves correctly.
     """
@@ -102,9 +128,9 @@ def save_and_compare(arbor):
     fn = arbor.save_arbor()
     save_arbor = load(fn)
     assert isinstance(save_arbor, YTreeArbor)
-    compare_arbors(save_arbor, arbor)
+    compare_arbors(save_arbor, arbor, skip=skip)
 
-def compare_arbors(a1, a2, groups=None, fields=None):
+def compare_arbors(a1, a2, groups=None, fields=None, skip=1):
     """
     Compare all fields for all trees in two arbors.
     """
@@ -118,8 +144,12 @@ def compare_arbors(a1, a2, groups=None, fields=None):
     for field in fields:
         assert (a1[field] == a2[field]).all()
 
-    for t1, t2 in zip(a1, a2):
-        compare_trees(t1, t2, groups=groups, fields=fields)
+    ntot = a1.size
+    pbar = get_pbar("Comparing trees", ntot)
+    for i in range(0, ntot, skip):
+        compare_trees(a1[i], a2[i], groups=groups, fields=fields)
+        pbar.update(i)
+    pbar.finish()
 
 def compare_trees(t1, t2, groups=None, fields=None):
     """
@@ -134,9 +164,12 @@ def compare_trees(t1, t2, groups=None, fields=None):
 
     for field in fields:
         for group in groups:
-            assert (t1[group, field] == t2[group, field]).all()
-    t1.clear_fields()
-    t2.clear_fields()
+            assert_array_equal(
+                t1[group, field], t2[group, field],
+                err_msg="Tree comparison failed for %s field: %s." %
+                (group, field))
+    t1.reset()
+    t2.reset()
 
 def compare_hdf5(fh1, fh2, compare=None, compare_groups=True,
                  **kwargs):
